@@ -1,11 +1,11 @@
 use core::borrow::Borrow;
 use core::fmt::{self, Debug};
 use core::iter::FromIterator;
-use core::ops::Index;
+use core::ops::{Index, RangeBounds};
 
 use crate::map_types::{
-    Entry, IntoIter, IntoKeys, IntoValues, Iter, IterMut, Keys, OccupiedEntry, VacantEntry, Values,
-    ValuesMut,
+    Entry, IntoIter, IntoKeys, IntoValues, Iter, IterMut, Keys, OccupiedEntry, OccupiedError,
+    Range, RangeMut, VacantEntry, Values, ValuesMut,
 };
 use crate::tree::{SgError, SgTree};
 
@@ -335,6 +335,11 @@ impl<K: Ord + Default, V: Default, const N: usize> SgMap<K, V, N> {
     /// * The old value if the map did have this key present (both the value and key are updated,
     /// this accommodates types that can be `==` without being identical).
     ///
+    /// ### Warning
+    ///
+    /// Unlike other APIs in this crate, the semantics and return type of this API are *NOT* the same as `BTreeMap`'s nightly [`try_insert`](https://doc.rust-lang.org/std/collections/struct.BTreeMap.html#method.try_insert).
+    /// For an equivalent, use [`try_insert_std`][`SgMap::try_insert_std`].
+    ///
     /// # Examples
     ///
     /// ```
@@ -368,9 +373,6 @@ impl<K: Ord + Default, V: Default, const N: usize> SgMap<K, V, N> {
     /// assert_eq!(map.try_insert(key - 1, "overwrite filler"), Ok(Some("filler")));
     /// ```
     ///
-    /// ### Warning
-    ///
-    /// Unlike other APIs in this crate, the semantics and return type of this API are NOT the same as `BTreeMap`'s nightly [`try_insert`](https://doc.rust-lang.org/std/collections/struct.BTreeMap.html#method.try_insert).
     pub fn try_insert(&mut self, key: K, val: V) -> Result<Option<V>, SgError>
     where
         K: Ord,
@@ -378,6 +380,42 @@ impl<K: Ord + Default, V: Default, const N: usize> SgMap<K, V, N> {
         self.bst.try_insert(key, val)
     }
 
+    /// Tries to insert a key-value pair into the map, and returns
+    /// a mutable reference to the value in the entry.
+    ///
+    /// If the map already had this key present, nothing is updated, and
+    /// an error containing the occupied entry and the value is returned.
+    ///
+    /// ### Warning
+    ///
+    /// The semantics and return type of this API match `BTreeMap`'s nightly [`try_insert`](https://doc.rust-lang.org/std/collections/struct.BTreeMap.html#method.try_insert), *NOT* the other `try_*` APIs in this crate.
+    /// For a fallible insert, use [`try_insert`][`SgMap::try_insert`].
+    ///
+    /// # Examples
+    ///
+    /// Basic usage:
+    ///
+    /// ```
+    /// use scapegoat::SgMap;
+    ///
+    /// let mut map = SgMap::<_, _, 10>::new();
+    /// assert_eq!(map.try_insert_std(37, "a").unwrap(), &"a");
+    ///
+    /// let err = map.try_insert_std(37, "b").unwrap_err();
+    /// assert_eq!(err.entry.key(), &37);
+    /// assert_eq!(err.entry.get(), &"a");
+    /// assert_eq!(err.value, "b");
+    /// ```
+    pub fn try_insert_std(&mut self, key: K, value: V) -> Result<&mut V, OccupiedError<'_, K, V, N>>
+    where
+        K: Ord,
+    {
+        match self.entry(key) {
+            Entry::Occupied(entry) => Err(OccupiedError { entry, value }),
+            Entry::Vacant(entry) => Ok(entry.insert(value)),
+        }
+    }
+
     /// Attempt to extend a collection with the contents of an iterator.
     ///
     /// # Examples
@@ -977,6 +1015,89 @@ impl<K: Ord + Default, V: Default, const N: usize> SgMap<K, V, N> {
             table: self,
         })
     }
+
+    /// Constructs a double-ended iterator over a sub-range of elements in the map.
+    /// The simplest way is to use the range syntax `min..max`, thus `range(min..max)` will
+    /// yield elements from min (inclusive) to max (exclusive).
+    /// The range may also be entered as `(Bound<T>, Bound<T>)`, so for example
+    /// `range((Excluded(4), Included(10)))` will yield a left-exclusive, right-inclusive
+    /// range from 4 to 10.
+    ///
+    /// # Panics
+    ///
+    /// Panics if range `start > end`.
+    /// Panics if range `start == end` and both bounds are `Excluded`.
+    ///
+    /// # Examples
+    ///
+    /// Basic usage:
+    ///
+    /// ```
+    /// use scapegoat::SgMap;
+    /// use core::ops::Bound::Included;
+    ///
+    /// let mut map = SgMap::<_, _, 10>::new();
+    /// map.insert(3, "a");
+    /// map.insert(5, "b");
+    /// map.insert(8, "c");
+    /// for (&key, &value) in map.range((Included(&4), Included(&8))) {
+    ///     println!("{}: {}", key, value);
+    /// }
+    /// assert_eq!(Some((&5, &"b")), map.range(4..).next());
+    /// ```
+    pub fn range<T, R>(&self, range: R) -> Range<'_, K, V, N>
+    where
+        T: Ord + ?Sized,
+        K: Borrow<T> + Ord,
+        R: RangeBounds<T>,
+    {
+        Range {
+            table: self,
+            node_idx_iter: self.bst.range_search(range).into_iter(),
+        }
+    }
+
+    /// Constructs a mutable single-ended iterator over a sub-range of elements in the map.
+    /// The simplest way is to use the range syntax `min..max`, thus `range(min..max)` will
+    /// yield elements from min (inclusive) to max (exclusive).
+    /// The range may also be entered as `(Bound<T>, Bound<T>)`, so for example
+    /// `range((Excluded(4), Included(10)))` will yield a left-exclusive, right-inclusive
+    /// range from 4 to 10.
+    ///
+    /// # Panics
+    ///
+    /// Panics if range `start > end`.
+    /// Panics if range `start == end` and both bounds are `Excluded`.
+    ///
+    /// # Examples
+    ///
+    /// Basic usage:
+    ///
+    /// ```
+    /// use scapegoat::SgMap;
+    ///
+    /// let mut map: SgMap<_, _, 10> = ["Alice", "Bob", "Carol", "Cheryl"]
+    ///     .iter()
+    ///     .map(|&s| (s, 0))
+    ///     .collect();
+    /// for (_, balance) in map.range_mut("B".."Cheryl") {
+    ///     *balance += 100;
+    /// }
+    /// for (name, balance) in &map {
+    ///     println!("{} => {}", name, balance);
+    /// }
+    ///
+    /// assert_eq!(map["Alice"], 0);
+    /// assert_eq!(map["Bob"], 100);
+    /// ```
+    pub fn range_mut<T, R>(&mut self, range: R) -> RangeMut<'_, K, V, N>
+    where
+        T: Ord + ?Sized,
+        K: Borrow<T> + Ord,
+        R: RangeBounds<T>,
+    {
+        RangeMut::new(self, &range)
+    }
 }
 
 // Convenience Traits --------------------------------------------------------------------------------------------------
