@@ -1,3 +1,10 @@
+use core::borrow::Borrow;
+use core::fmt;
+use core::iter::{FusedIterator, Peekable};
+use core::ops::RangeBounds;
+
+use tinyvec::ArrayVec;
+
 use crate::map::SgMap;
 use crate::tree::{
     Idx, IntoIter as TreeIntoIter, Iter as TreeIter, IterMut as TreeIterMut, SmallNode,
@@ -37,6 +44,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> ExactSizeIterator for Ite
     }
 }
 
+impl<'a, K: Ord + Default, V: Default, const N: usize> FusedIterator for Iter<'a, K, V, N> {}
+
 /// An owning iterator over the entries of a [`SgMap`][crate::map::SgMap].
 ///
 /// This `struct` is created by the [`into_iter`][crate::map::SgMap::into_iter] method on [`SgMap`][crate::map::SgMap].
@@ -68,6 +77,8 @@ impl<K: Ord + Default, V: Default, const N: usize> ExactSizeIterator for IntoIte
     }
 }
 
+impl<K: Ord + Default, V: Default, const N: usize> FusedIterator for IntoIter<K, V, N> {}
+
 /// An mutable iterator over the entries of a [`SgMap`][crate::map::SgMap].
 ///
 /// This `struct` is created by the [`iter_mut`][crate::map::SgMap::iter_mut] method on [`SgMap`][crate::map::SgMap].
@@ -99,6 +110,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> ExactSizeIterator for Ite
     }
 }
 
+impl<'a, K: Ord + Default, V: Default, const N: usize> FusedIterator for IterMut<'a, K, V, N> {}
+
 // Key Iterators -------------------------------------------------------------------------------------------------------
 
 // TODO: these need more trait implementations for full compatibility
@@ -125,6 +138,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> ExactSizeIterator for Key
     }
 }
 
+impl<'a, K: Ord + Default, V: Default, const N: usize> FusedIterator for Keys<'a, K, V, N> {}
+
 /// An owning iterator over the keys of a [`SgMap`][crate::map::SgMap].
 ///
 /// This `struct` is created by the [`into_keys`][crate::map::SgMap::into_keys] method on [`SgMap`][crate::map::SgMap].
@@ -147,6 +162,8 @@ impl<K: Ord + Default, V: Default, const N: usize> ExactSizeIterator for IntoKey
     }
 }
 
+impl<K: Ord + Default, V: Default, const N: usize> FusedIterator for IntoKeys<K, V, N> {}
+
 // Value Iterators -----------------------------------------------------------------------------------------------------
 
 // TODO: these need more trait implementations for full compatibility
@@ -173,6 +190,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> ExactSizeIterator for Val
     }
 }
 
+impl<'a, K: Ord + Default, V: Default, const N: usize> FusedIterator for Values<'a, K, V, N> {}
+
 /// An owning iterator over the values of a [`SgMap`][crate::map::SgMap].
 ///
 /// This `struct` is created by the [`into_values`][crate::map::SgMap::into_values] method on [`SgMap`][crate::map::SgMap].
@@ -195,6 +214,8 @@ impl<K: Ord + Default, V: Default, const N: usize> ExactSizeIterator for IntoVal
     }
 }
 
+impl<K: Ord + Default, V: Default, const N: usize> FusedIterator for IntoValues<K, V, N> {}
+
 /// A mutable iterator over the values of a [`SgMap`][crate::map::SgMap].
 ///
 /// This `struct` is created by the [`values_mut`][crate::map::SgMap::values_mut] method on [`SgMap`][crate::map::SgMap].
@@ -219,6 +240,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> ExactSizeIterator
     }
 }
 
+impl<'a, K: Ord + Default, V: Default, const N: usize> FusedIterator for ValuesMut<'a, K, V, N> {}
+
 // Entry APIs ----------------------------------------------------------------------------------------------------------
 
 /// A view into a single entry in a map, which may either be vacant or occupied.
@@ -231,8 +254,6 @@ pub enum Entry<'a, K: Ord + Default, V: Default, const N: usize> {
     Occupied(OccupiedEntry<'a, K, V, N>),
 }
 
-use Entry::*;
-
 impl<'a, K: Ord + Default, V: Default, const N: usize> Entry<'a, K, V, N> {
     /// Ensures a value is in the entry by inserting the default if empty, and returns a mutable
     /// reference to the value in the entry.
@@ -249,8 +270,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> Entry<'a, K, V, N> {
     /// ```
     pub fn or_insert(self, default: V) -> &'a mut V {
         match self {
-            Occupied(entry) => entry.into_mut(),
-            Vacant(entry) => entry.insert(default),
+            Entry::Occupied(entry) => entry.into_mut(),
+            Entry::Vacant(entry) => entry.insert(default),
         }
     }
 
@@ -270,8 +291,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> Entry<'a, K, V, N> {
     /// ```
     pub fn or_insert_with<F: FnOnce() -> V>(self, default: F) -> &'a mut V {
         match self {
-            Occupied(entry) => entry.into_mut(),
-            Vacant(entry) => entry.insert(default()),
+            Entry::Occupied(entry) => entry.into_mut(),
+            Entry::Vacant(entry) => entry.insert(default()),
         }
     }
 
@@ -295,8 +316,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> Entry<'a, K, V, N> {
     /// ```
     pub fn or_insert_with_key<F: FnOnce(&K) -> V>(self, default: F) -> &'a mut V {
         match self {
-            Occupied(entry) => entry.into_mut(),
-            Vacant(entry) => {
+            Entry::Occupied(entry) => entry.into_mut(),
+            Entry::Vacant(entry) => {
                 let value = default(entry.key());
                 entry.insert(value)
             }
@@ -315,8 +336,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> Entry<'a, K, V, N> {
     /// ```
     pub fn key(&self) -> &K {
         match self {
-            Occupied(entry) => entry.key(),
-            Vacant(entry) => entry.key(),
+            Entry::Occupied(entry) => entry.key(),
+            Entry::Vacant(entry) => entry.key(),
         }
     }
 
@@ -342,11 +363,11 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> Entry<'a, K, V, N> {
     /// ```
     pub fn and_modify<F: FnOnce(&mut V)>(self, f: F) -> Entry<'a, K, V, N> {
         match self {
-            Occupied(mut entry) => {
+            Entry::Occupied(mut entry) => {
                 f(entry.get_mut());
-                Occupied(entry)
+                Entry::Occupied(entry)
             }
-            Vacant(entry) => Vacant(entry),
+            Entry::Vacant(entry) => Entry::Vacant(entry),
         }
     }
 
@@ -365,8 +386,8 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> Entry<'a, K, V, N> {
     /// ```
     pub fn or_default(self) -> &'a mut V {
         match self {
-            Occupied(entry) => entry.into_mut(),
-            Vacant(entry) => entry.insert(Default::default()),
+            Entry::Occupied(entry) => entry.into_mut(),
+            Entry::Vacant(entry) => entry.insert(Default::default()),
         }
     }
 }
@@ -600,3 +621,198 @@ impl<'a, K: Ord + Default, V: Default, const N: usize> OccupiedEntry<'a, K, V, N
         self.remove_entry().1
     }
 }
+
+/// The error returned by [`try_insert_std`](SgMap::try_insert_std) when the key already exists.
+///
+/// Contains the occupied entry, and the value that was not inserted.
+pub struct OccupiedError<'a, K: 'a + Ord + Default, V: 'a + Default, const N: usize> {
+    /// The entry in the map that was already occupied.
+    pub entry: OccupiedEntry<'a, K, V, N>,
+    /// The value which was not inserted, because the entry was already occupied.
+    pub value: V,
+}
+
+impl<K: fmt::Debug + Ord + Default, V: fmt::Debug + Default, const N: usize> fmt::Debug
+    for OccupiedError<'_, K, V, N>
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("OccupiedError")
+            .field("key", self.entry.key())
+            .field("old_value", self.entry.get())
+            .field("new_value", &self.value)
+            .finish()
+    }
+}
+
+impl<'a, K: fmt::Debug + Ord + Default, V: fmt::Debug + Default, const N: usize> fmt::Display
+    for OccupiedError<'a, K, V, N>
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(
+            f,
+            "failed to insert {:?}, key {:?} already exists with value {:?}",
+            self.value,
+            self.entry.key(),
+            self.entry.get(),
+        )
+    }
+}
+
+// Range APIs ----------------------------------------------------------------------------------------------------------
+
+/// An iterator over a sub-range of entries in a [`SgMap`].
+///
+/// This `struct` is created by the [`range`][`crate::map::SgMap::range`] method on [`SgMap`][crate::map::SgMap]. See its
+/// documentation for more.
+pub struct Range<'a, K: Ord + Default, V: Default, const N: usize> {
+    pub(crate) table: &'a SgMap<K, V, N>,
+    pub(crate) node_idx_iter: <ArrayVec<[usize; N]> as IntoIterator>::IntoIter,
+}
+
+impl<'a, K: Ord + Default, V: Default, const N: usize> Range<'a, K, V, N> {
+    fn to_node_ref(&self, idx: usize) -> (&'a K, &'a V) {
+        let node = &self.table.bst.arena[idx];
+        (node.key(), node.val())
+    }
+}
+
+impl<'a, K: Ord + Default, V: Default, const N: usize> Iterator for Range<'a, K, V, N> {
+    type Item = (&'a K, &'a V);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let node_idx = self.node_idx_iter.next()?;
+        Some(self.to_node_ref(node_idx))
+    }
+}
+
+impl<'a, K: Ord + Default, V: Default, const N: usize> DoubleEndedIterator for Range<'a, K, V, N> {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        let node_idx = self.node_idx_iter.next_back()?;
+        Some(self.to_node_ref(node_idx))
+    }
+}
+
+impl<'a, K: Ord + Default, V: Default, const N: usize> FusedIterator for Range<'a, K, V, N> {}
+
+/// A mutable iterator over a sub-range of entries in a [`SgMap`].
+///
+/// This `struct` is created by the [`range_mut`] method on [`SgMap`]. See its
+/// documentation for more.
+///
+/// [`range_mut`]: SgMap::range_mut
+pub struct RangeMut<'a, K: Ord + Default, V: Default, const N: usize> {
+    inner: Peekable<TreeIterMut<'a, K, V, N>>,
+    total_cnt: usize,
+    spent_cnt: usize,
+}
+
+impl<'a, K, V, const N: usize> RangeMut<'a, K, V, N>
+where
+    K: Ord + Default,
+    V: Default,
+{
+    // Constructor
+    pub(crate) fn new<T, R>(map: &'a mut SgMap<K, V, N>, range: &R) -> Self
+    where
+        T: Ord + ?Sized,
+        K: Borrow<T> + Ord + Default,
+        R: RangeBounds<T>,
+    {
+        let len = RangeMut::compute_len(map, range);
+
+        Self {
+            inner: RangeMut::init_iter_mut(map, range),
+            total_cnt: len,
+            spent_cnt: 0,
+        }
+    }
+
+    // Compute amount of items to return
+    fn compute_len<T, R>(map: &SgMap<K, V, N>, range: &R) -> usize
+    where
+        T: Ord + ?Sized,
+        K: Borrow<T> + Ord + Default,
+        R: RangeBounds<T>,
+    {
+        let mut peekable = map.bst.iter().peekable();
+        let mut len = 0;
+
+        // Advance immutable iter to start
+        while let Some(node) = peekable.peek() {
+            if range.contains(node.0.borrow()) {
+                break;
+            }
+
+            peekable.next();
+        }
+
+        // Count remaining
+        for node in peekable {
+            if range.contains(node.0.borrow()) {
+                len += 1;
+            } else {
+                break;
+            }
+        }
+
+        len
+    }
+
+    // Prepare mutable iterator to return first item in range
+    fn init_iter_mut<T, R>(
+        map: &'a mut SgMap<K, V, N>,
+        range: &R,
+    ) -> Peekable<TreeIterMut<'a, K, V, N>>
+    where
+        T: Ord + ?Sized,
+        K: Borrow<T> + Ord + Default,
+        R: RangeBounds<T>,
+    {
+        let mut peekable = map.bst.iter_mut().peekable();
+
+        // Advance mutable iter to start
+        while let Some(node) = peekable.peek() {
+            if range.contains(node.0.borrow()) {
+                break;
+            }
+
+            peekable.next();
+        }
+
+        peekable
+    }
+}
+
+impl<'a, K, V, const N: usize> Iterator for RangeMut<'a, K, V, N>
+where
+    K: Ord + Default,
+    V: Default,
+{
+    type Item = (&'a K, &'a mut V);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.spent_cnt < self.total_cnt {
+            let node = self.inner.next()?;
+            self.spent_cnt += 1;
+            Some(node)
+        } else {
+            None
+        }
+    }
+}
+
+impl<'a, K: Ord + Default, V: Default, const N: usize> FusedIterator for RangeMut<'a, K, V, N> {}
+
+/*
+// TODO: does commit to this interface limit potential optimizations?
+impl<'a, K, V, const N: usize> ExactSizeIterator for RangeMut<'a, K, V, N>
+where
+    K: Ord + Default,
+    V: Default,
+{
+    fn len(&self) -> usize {
+        debug_assert!(self.spent_cnt <= self.total_cnt);
+        self.total_cnt - self.spent_cnt
+    }
+}
+*/
