@@ -4,6 +4,7 @@ use core::fmt::{self, Debug};
 use core::hash::{Hash, Hasher};
 use core::iter::FromIterator;
 use core::mem;
+use core::ops::RangeBounds;
 use core::ops::{Index, Sub};
 
 use super::arena::Arena;
@@ -529,6 +530,30 @@ impl<K: Ord + Default, V: Default, const N: usize> SgTree<K, V, N> {
         Idx::MAX as usize
     }
 
+    pub(crate) fn range_search<T, R>(&self, range: R) -> ArrayVec<[usize; N]>
+    where
+        T: Ord + ?Sized,
+        R: RangeBounds<T>,
+        K: Borrow<T> + Ord,
+    {
+        let mut node_idxs = ArrayVec::<[usize; N]>::new();
+
+        for (idx, node) in self
+            .arena
+            .iter()
+            .enumerate()
+            .filter_map(|(i, node)| Some((i, node.as_ref()?)))
+        {
+            if range.contains(node.key().borrow()) {
+                node_idxs.push(idx);
+            }
+        }
+
+        node_idxs.sort_unstable_by_key(|idx| self.arena[*idx].key());
+
+        node_idxs
+    }
+
     // Private API -----------------------------------------------------------------------------------------------------
 
     // Iterative search. If key found, returns node idx, parent idx, and a bool indicating if node is right child
